<?php

namespace ReadyApp\Exactoris\UI\View\Form;

use Application;
use DialogForm;
use DialogComposer;
use ExtSelect;
use ExcelEditorWidget;
use InvalidParamException;
use Panel;
use PostPanelTab;
use NewPostPanel;
use TextArea2;
use TextInput2;
use TextAreaEx;
use ReadyApp\Exactoris\Application\Service\Translator;
use ReadyApp\Exactoris\Application\Service\ScenarioService;
use ReadyApp\Exactoris\Application\States\States;
use ReadyApp\Exactoris\Exception\Exception;
use ReadyApp\Exactoris\UI\View\Helper\JsHelper;

/**
 * Class ScenarioDialog
 * Created by Scafold.
 *
 * @version          0.0.1
 * @copyright (c)    eDokumenty Sp. z o.o.
 */
class ScenarioDialog extends DialogForm {

    protected static array $cMethods = [
        'asyncRemoveScenario',
        'confirmCloseScenario',
    ];


    private string    $HWND;
    private ExtSelect $status;
    private bool      $forRead = FALSE;

    /**
     * @param $name
     * @param $caption
     * @param $dstyle
     *
     * @throws InvalidParamException
     * @throws \SweetException
     */
    public function __construct($name, $caption = NULL, $dstyle = BS_DIALOG) {

        parent::__construct($name, 'Scenariusz', $dstyle);
        $this->path = Application::makeRelativePath(__FILE__);
        $this->HWND = Application::registerDialog(Application::getShortName(__CLASS__), __CLASS__, $this->path);

        $this->width = '850px';
        $this->height = '700px';

        $this->sizeable = TRUE;
        $this->icon = 'eic eic-form-filled';
        if (($this->isOpened()) && (!$this->noNeedForCreate())) {
            $this->create();
        }
    }

    public function getSupportedParams(): array {
        return [
            'id____' => TRUE,
            'mode'   => TRUE,
            'JsScript' => TRUE
        ];
    }

    /**
     * getCallableMethods
     *
     * @access public
     * @return array
     */
    public function getCallableMethods(): array {
        return self::$cMethods;
    }

    /**
     * @param string $methodName
     *
     * @return int
     */
    public static function getMethodID(string $methodName): int {
        return array_search($methodName, self::$cMethods, TRUE);
    }

    /**
     * @param $params
     *
     * @return void
     * @throws \SweetException
     * @throws \WidgetException
     */
    public function asyncRemoveScenario($params) {
        $data = \JSON::toArray($params);

        $id = (int)$data['id____'] ?? NULL;
        if (bs_empty_str($id)) {
            throw new \WidgetException(NULL, Translator::translate('Prosze zaznaczyć scenariusz!'));
        }

        try {
            $service = new ScenarioService();
            $service->deleteById($id);
        } catch (Exception $e) {
            throw $e->getSweetException();
        }

        \JScript::notifySuccess('Scenariusz został usunięty');
    }

    /**
     * @param $params
     * @param $data
     *
     * @return void
     * @throws InvalidParamException
     * @throws \CustomErrorException
     * @throws \SweetException
     */
    public function open($params = NULL, &$data = NULL) {
        $this->clear();
        $pData = \JSON::toArray($params);

        parent::open($params, $data);

        $mode = $data['mode'] ?? NULL;
        $id____ = $data['id____'] ?? NULL;
        $prc_id = $pData['prc_id'] ?? NULL;

        if (!in_array($mode, ['new', 'edit', 'forRead','newFromExists', 'delete'])) {
            throw new \CustomErrorException('Dialog mode not supported!');
        }

        $service = new ScenarioService();
        if ($mode === 'delete') {
            $this->close();
            try {
                $model = $service->getModelById($id____);
            } catch (Exception $e) {
                throw $e->getSweetException();
            }

            $onOKClick = JsHelper::prepareA1Function($this->HWND, self::getMethodID('asyncRemoveScenario'), '{id____: '.$model->getId().'}', $pData['afterSubmit'] ?? '');
            $format = 'Czy na pewno chcesz usunąć scenariusz %s?';
            \JScript::confirmDelete(sprintf($format, '<b>'.$model->title.'</b>'), $onOKClick);

            return;
        }

        if ($id____) {
            $statusId = $service->getModelById($id____)->statusId;
            $data['tpstid'] = $statusId;

            if ($statusId === States::getStatusId(States::SCENARIO_VERIFIED)) {
                $mode = 'forRead';
            }
        }


        $data['prc_id'] = $prc_id;

        $this->forRead = $mode === 'forRead';
        $data['headerTitle'] = Translator::translate('Utwórz nowy scenariusz');
        if ($mode === 'forRead') {
            $data['headerTitle'] = Translator::translate('Podgląd scenariusza');
        }
        $this->setSData($data);
        $this->create();

        try {
            if ($mode === 'new') {
                $model = $service->getEntityWithDefaultValues();
            } else {
                if ($id____ === NULL) {
                    throw new \WidgetException(NULL, Translator::translate('Proszę zaznaczyć scenariusz!'));
                }
                $model = $service->getModelById($id____);
            }
        } catch (Exception $e) {
            throw $e->getSweetException();
        }

        $this->setWData(array_filter($model->toArray()));
        if ($mode === 'newFromExists') {
            unset($data['id____']);
            $data['mode'] = 'new';
            $_SESSION[$this->name]['_keyval'] = null;
        }


        $this->setSData($data);
    }

    /**
     * @return bool
     * @throws \JsonException
     * @throws \SweetException
     * @throws \WidgetException
     */
    public function save() {
        if (!$this->isOpened()) {
            return FALSE;
        }

        $sdata = $this->getSData();
        $id____ = $sdata['id____'] ?? NULL;

        $data = $this->getWData();
        $this->validate();

        try {
            /**
             * Sekcja do zapisu
             */

            \JScript::notifySuccess(Translator::translate('Scenariusz został zapisany!'));

        } catch (Exception $e) {
            throw $e->getSweetException();
        }

        if ($data['to_close'] === 'TRUE') {
            $this->jsClose();
        }


        return parent::save();
    }

    /**
     * App.openDialogByCls('CUSTOM_EXACTORIS_EDIT_SCENARIOS', 1020, ({mode:"forRead"}).toJSONString());
     *
     * @return void
     * @throws InvalidParamException
     * @throws \SweetException
     */
    private function create(): void {
        if (!$this->isOpened()) {
            return;
        }
        $data = $this->getSData();

        $title = new TextInput2($this->name.'Title');
        $title->setLabel(Translator::translate('Tytuł'));
        $title->setRequired();
        $title->setReadOnly($this->forRead);
        $title->width = 'calc(100% - '.(self::SPACE_LEFT + self::SPACE_RIGHT).'px)';


        $dscrpt = new TextArea2($this->name.'Description');
        $dscrpt->setLabel(Translator::translate('Opis'));
        $dscrpt->setReadOnly($this->forRead);
        $dscrpt->height = (self::DEFAULT_WIDGET_HEIGHT * 3).'px';


        $tabPanel = new NewPostPanel($this->name.'_NPP', NewPostPanel::STYLE5);
        $tabPanel->left = '0px';
        $tabPanel->width = '100%';
        $tabPanel->height = 'calc(100% - 200px) !important';
        $tabPanel->position = 'relative';
        $tabPanel->pageStyle = 'height:calc(100% - 32px); padding:0;';
        $tabPanel->style = 'padding:0px; box-sizing:border-box; -moz-box-sizing: border-box;';

        $informationPanel = $tabPanel->addTab('Information', Translator::translate('Informacje'),!$this->forRead);
        $this->createInformationPanel($informationPanel);

        $preConditionPanel = $tabPanel->addTab('preCondition', Translator::translate('Warunki wstępne'));
        $this->createPreConditionPanel($preConditionPanel);

        $scenarioPanel = $tabPanel->addTab('Scenario', Translator::translate('Scenariusz'), $this->forRead);
        $excelEditor = $this->createExcelEditorWidget();
        $scenarioPanel->addNext($excelEditor, 'content');

        $composer = new DialogComposer($this);
        if ($data['mode'] !== 'forRead') {
            $composer->addModernHeader($data['headerTitle']);
        }

        $composer->addNext($title, 'title');
        $composer->addNext($dscrpt, 'description');
        $composer->addNext($tabPanel);

        $composer->addModernButtonsPanel(['bSave','bOk', 'bCancel']);

        $toClose = new \CheckBox2($this->name.'ToClose', false, TRUE);
        $toClose->style = 'display:none;';
        $composer->add($toClose, 'to_close');

        $this->bOk->caption = Translator::translate('Zapisz i zamknij');
        $this->onbOkClick = 'App.'.$this->name.'SetAutoCloseAfterSave(true);App.'.$this->name.'.asyncSubmit(\''.$this->bSave->name.'\',Dialog.saveCallback); return false;';
        $this->onbSaveClick = 'App.'.$this->name.'SetAutoCloseAfterSave(false);';

        $this->bCancel->onclick = 'App.'.$this->name.'confirmClose(); doNil(event); return false;';
        $this->add($composer);

        \JScript::add('
        App.'.$this->name.'SetAutoCloseAfterSave = function (toClose) {
            field = FormInputHelper.create(\''.$toClose->name.'\');
            field.setValue(toClose);
        };
        App.'.$this->name.'confirmClose = function () {
            '.\JScript::confirmAction([
                'html' => Translator::translate('Czy na pewno chcesz zamknąć scenariusz bez zapisu?'),
                'confirmButtonText' => Translator::translate('tak'),
                'cancelButtonText' => Translator::translate('nie'),
            ],'App.'.$this->name.'.close();',NULL,FALSE).';
        }
        ');
    }

    /**
     * @return ExcelEditorWidget
     * @throws InvalidParamException
     */
    private function createExcelEditorWidget(): ExcelEditorWidget {
        require_once(LIB_PATH.'forms/ExcelEditorWidget.inc');
        $editor = new ExcelEditorWidget($this->name.'scenarioStepsWithResults', 2);
        $editor->addColumn('Kroki');
        $editor->addColumn('Oczekiwany rezultat');
        $editor->setReadOnly($this->forRead);

        $editor->top = self::SPACE_TOP.'px';
        $editor->left = self::SPACE_LEFT.'px';
        $editor->width = 'calc(100% - '.(self::SPACE_RIGHT + self::SPACE_LEFT).'px)';
        $editor->height = 'calc(100% - '.(self::SPACE_RIGHT + self::SPACE_LEFT).'px)';

        return $editor;
    }

    /**
     * @param PostPanelTab $informationPanel
     *
     * @return void
     * @throws \SweetException
     */
    private function createInformationPanel(PostPanelTab $informationPanel): void {

        $column = new Panel($this->name.'Column');
        $column->width = 'calc(33% - '.(self::SPACE_LEFT + self::SPACE_RIGHT / 2).'px)';

        $title = new TextInput2($this->name.'Title');
        $title->setLabel(Translator::translate('Tytuł scenariusza'));
        $title->setRequired();
        $title->setReadOnly($this->forRead);
        $title->width = 'calc(100% - '.(self::SPACE_LEFT + self::SPACE_RIGHT).'px)';

        $dscrpt = new TextArea2($this->name.'Description');
        $dscrpt->setLabel(Translator::translate('Opis'));
        $dscrpt->setReadOnly($this->forRead);
        $dscrpt->height = (self::DEFAULT_WIDGET_HEIGHT * 3).'px';

        $column->addNext($title, 'title');
        $column->addNext($dscrpt, 'dscrpt');

        $informationPanel->addNext($column);
    }

    /**
     * @param PostPanelTab $preConditionPanel
     *
     * @return void
     */
    private function createPreConditionPanel(PostPanelTab $preConditionPanel): void {
        $listPanel = new Panel($this->name.'Panel');
        $listPanel->height = 'calc(100% - '.(DialogComposer::BUTTONS_PANEL_HEIGHT).'px)';
        $listPanel->width = 'calc(100% - '.(self::SPACE_RIGHT + self::SPACE_LEFT).'px)';

        $editor = new TextAreaEx($this->name.'PreConditionEditor');
        $editor->width  = '100%';
        $editor->height = '100%';
        $editor->setLabel(Translator::translate('Warunki wstępne'));
        $editor->setReadOnly($this->forRead);
        $listPanel->add($editor, 'preconditions');
        $preConditionPanel->addNext($listPanel);
    }

    /**
     * @return void
     * @throws \WidgetException
     */
    private function validate() {
        $data = $this->getWData();

        if (bs_empty_str($data['title'])) {
            throw new \WidgetException($this->name.'Title', Translator::translate('Pole wymagane!'));
        }
    }
}